package com.ruoyi.system.controller;

import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.PostMapping;
import com.ruoyi.common.log.enums.BusinessType;
import com.ruoyi.common.log.annotation.Log;
import com.ruoyi.common.security.utils.SecurityUtils;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.validation.annotation.Validated;
import org.apache.commons.lang3.ArrayUtils;
import com.ruoyi.common.core.utils.StringUtils;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.GetMapping;
import com.ruoyi.common.security.annotation.RequiresPermissions;
import java.util.List;
import com.ruoyi.common.core.web.domain.AjaxResult;
import com.ruoyi.system.api.domain.SysDept;
import org.springframework.beans.factory.annotation.Autowired;
import com.ruoyi.system.service.ISysDeptService;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.core.web.controller.BaseController;

@RestController
@RequestMapping({ "/dept" })
public class SysDeptController extends BaseController
{
    @Autowired
    private ISysDeptService deptService;
    
    @RequiresPermissions({ "system:dept:list" })
    @GetMapping({ "/list" })
    public AjaxResult list(final SysDept dept) {
        final List<SysDept> depts = this.deptService.selectDeptList(dept);
        return this.success((Object)depts);
    }
    
    @RequiresPermissions({ "system:dept:list" })
    @GetMapping({ "/list/exclude/{deptId}" })
    public AjaxResult excludeChild(@PathVariable(value = "deptId", required = false) final Long deptId) {
        final List<SysDept> depts = this.deptService.selectDeptList(new SysDept());
        depts.removeIf(d -> {
            final boolean b;
            if (d.getDeptId().intValue() != deptId) {
                if (!ArrayUtils.contains(StringUtils.split(d.getAncestors(), ","), deptId + "")) {
                    return b;
                }
            }
            return b;
        });
        return this.success((Object)depts);
    }
    
    @RequiresPermissions({ "system:dept:query" })
    @GetMapping({ "/{deptId}" })
    public AjaxResult getInfo(@PathVariable final Long deptId) {
        this.deptService.checkDeptDataScope(deptId);
        return this.success((Object)this.deptService.selectDeptById(deptId));
    }
    
    @RequiresPermissions({ "system:dept:add" })
    @Log(title = "部门管理", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@Validated @RequestBody final SysDept dept) {
        if (!this.deptService.checkDeptNameUnique(dept)) {
            return this.error("新增部门'" + dept.getDeptName() + "'失败，部门名称已存在");
        }
        dept.setCreateBy(SecurityUtils.getUsername());
        return this.toAjax(this.deptService.insertDept(dept));
    }
    
    @RequiresPermissions({ "system:dept:edit" })
    @Log(title = "部门管理", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@Validated @RequestBody final SysDept dept) {
        final Long deptId = dept.getDeptId();
        this.deptService.checkDeptDataScope(deptId);
        if (!this.deptService.checkDeptNameUnique(dept)) {
            return this.error("修改部门'" + dept.getDeptName() + "'失败，部门名称已存在");
        }
        if (dept.getParentId().equals(deptId)) {
            return this.error("修改部门'" + dept.getDeptName() + "'失败，上级部门不能是自己");
        }
        if (StringUtils.equals((CharSequence)"1", (CharSequence)dept.getStatus()) && this.deptService.selectNormalChildrenDeptById(deptId) > 0) {
            return this.error("该部门包含未停用的子部门！");
        }
        dept.setUpdateBy(SecurityUtils.getUsername());
        return this.toAjax(this.deptService.updateDept(dept));
    }
    
    @RequiresPermissions({ "system:dept:remove" })
    @Log(title = "部门管理", businessType = BusinessType.DELETE)
    @DeleteMapping({ "/{deptId}" })
    public AjaxResult remove(@PathVariable final Long deptId) {
        if (this.deptService.hasChildByDeptId(deptId)) {
            return this.warn("存在下级部门,不允许删除");
        }
        if (this.deptService.checkDeptExistUser(deptId)) {
            return this.warn("部门存在用户,不允许删除");
        }
        this.deptService.checkDeptDataScope(deptId);
        return this.toAjax(this.deptService.deleteDeptById(deptId));
    }
}
